CANCELLED_APPT = "cancelled"
CLINIC = "clinic"
COMPLETE_APPT = "done"
INCOMPLETE_APPT = "incomplete"
IN_PROGRESS_APPT = "in_progress"
LATE_APPT = "late"
MISSED_APPT = "missed"
NEW_APPT = "new"
ONTIME_APPT = "ontime"
SCHEDULED_APPT = "scheduled"
UNSCHEDULED_APPT = "unscheduled"
