# Generated by Django 2.0rc1 on 2017-11-28 10:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_appointment", "0010_auto_20171124_1120")]

    operations = [
        migrations.AlterField(
            model_name="appointment",
            name="appt_close_datetime",
            field=models.DateTimeField(
                help_text="timepoint_datetime adjusted according to the nearest available datetime for this facility",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="appointment",
            name="timepoint",
            field=models.DecimalField(
                decimal_places=1,
                help_text="timepoint from schedule",
                max_digits=6,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="appointment",
            name="timepoint_datetime",
            field=models.DateTimeField(
                help_text="Unadjusted datetime calculated from visit schedule",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="appt_close_datetime",
            field=models.DateTimeField(
                help_text="timepoint_datetime adjusted according to the nearest available datetime for this facility",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="timepoint",
            field=models.DecimalField(
                decimal_places=1,
                help_text="timepoint from schedule",
                max_digits=6,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="historicalappointment",
            name="timepoint_datetime",
            field=models.DateTimeField(
                help_text="Unadjusted datetime calculated from visit schedule",
                null=True,
            ),
        ),
    ]
