# Generated by Django 4.0.4 on 2022-05-19 20:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('edc_appointment', '0027_appointment_document_status_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='appointment',
            name='document_status_comments',
            field=models.TextField(blank=True, help_text='for example, why some data is still pending', null=True, verbose_name='Any comments related to status of this document'),
        ),
        migrations.AddField(
            model_name='historicalappointment',
            name='document_status_comments',
            field=models.TextField(blank=True, help_text='for example, why some data is still pending', null=True, verbose_name='Any comments related to status of this document'),
        ),
    ]
