# Generated by Django 4.0.4 on 2022-05-19 19:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('edc_appointment', '0026_auto_20220501_1548'),
    ]

    operations = [
        migrations.AddField(
            model_name='appointment',
            name='document_status',
            field=models.CharField(choices=[('INCOMPLETE', 'Incomplete (some data pending)'), ('COMPLETE', 'Complete')], default='COMPLETE', help_text='If some data is still pending, flag as incomplete', max_length=25, verbose_name='Document status'),
        ),
        migrations.AddField(
            model_name='historicalappointment',
            name='document_status',
            field=models.CharField(choices=[('INCOMPLETE', 'Incomplete (some data pending)'), ('COMPLETE', 'Complete')], default='COMPLETE', help_text='If some data is still pending, flag as incomplete', max_length=25, verbose_name='Document status'),
        ),
    ]
